/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 *  Copyright (c) 2011 Oracle and/or its affiliates. All rights reserved.
 *
 *  The contents of this file are subject to the terms of either the GNU
 *  General Public License Version 2 only ("GPL") or the Common Development
 *  and Distribution License("CDDL") (collectively, the "License").  You
 *  may not use this file except in compliance with the License.  You can
 *  obtain a copy of the License at
 *  https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 *  or packager/legal/LICENSE.txt.  See the License for the specific
 *  language governing permissions and limitations under the License.
 *
 *  When distributing the software, include this License Header Notice in each
 *  file and include the License file at packager/legal/LICENSE.txt.
 *
 *  GPL Classpath Exception:
 *  Oracle designates this particular file as subject to the "Classpath"
 *  exception as provided by Oracle in the GPL Version 2 section of the License
 *  file that accompanied this code.
 *
 *  Modifications:
 *  If applicable, add the following below the License Header, with the fields
 *  enclosed by brackets [] replaced by your own identifying information:
 *  "Portions Copyright [year] [name of copyright owner]"
 *
 *  Contributor(s):
 *  If you wish your version of this file to be governed by only the CDDL or
 *  only the GPL Version 2, indicate your decision by adding "[Contributor]
 *  elects to include this software in this distribution under the [CDDL or GPL
 *  Version 2] license."  If you don't indicate a single choice of license, a
 *  recipient has the option to distribute your version of this file under
 *  either the CDDL, the GPL Version 2 or to extend the choice of license to
 *  its licensees as provided above.  However, if you add GPL Version 2 code
 *  and therefore, elected the GPL Version 2 license, then the option applies
 *  only if the new code is made subject to such option by the copyright
 *  holder.
 */
package org.glassfish.virtualization.spi;

import org.glassfish.virtualization.config.Template;

import java.util.*;

/**
 * Virtual machine procurement order.
 * @author Jerome Dochez
 */
public class VMOrder {

    final int number;
    final Template template;
    final List<Group> groups = new ArrayList<Group>();
    final List<VirtualMachine> noColocationList = new ArrayList<VirtualMachine>();
    final Properties vmProps = new Properties();

    public VMOrder(Template template, int number) {
        this.number = number;
        this.template = template;
    }

    /**
     *
     * @return the desired number of machines
     */
    public int number() {
        return number;
    }

    /**
     * Returns the requested characteristics for the virtual machines
     * @return a set of static virtual machine characteristics to use when
     * allocating the new virtual machines or null if the template characteristics
     * should apply.
     */
    public StaticVirtualMachineInfo getCharacteristics() {
        return null;
    }

    /**
     * Specifies the group in which the number of virtual machines should be allocated.
     * If no group is specified, it's left to the Infrastructure Management Service to
     * decide in which groups those Virtual Machines will be allocated.
     *
     * @param groups desired group instance
     * @return itself
     */
    public VMOrder in(Group... groups) {
        this.groups.addAll(Arrays.asList(groups));
        return this;
    }

    /**
     * Specifies the virtual machines that should not be co-located on the same hardware
     * with the new allocated virtual machines. This is particularly useful when willing
     * to allocate replication instances for existing virtual machines so they do not end
     * up being running on the same hardware resource.
     *
     * @param vms list of virtual machines to not co-locate with.
     * @return itself.
     */
    public VMOrder noColocationWith(VirtualMachine... vms) {
        this.noColocationList.addAll(Arrays.asList(vms));
        return this;
    }

    /**
     * Returns the properties for a specific virtual machine allocation. These properties
     * will be passed to the virtual machine through a provider specific mechanism. Such
     * properties can be used by the virtual machine to configure itself.
     *
     * @return the properties or an empty collection
     */
    public Properties getVirtualMachineProperties(int machineNumber) {
        return vmProps;
    }

    /**
     * Returns the groups this set of virtual machine allocations should be allocated into
     *
     * @return the groups we should use to allocate the virtual machines if possible.
     */
    public Collection<Group> affinities() {
        return Collections.unmodifiableList(groups);
    }

    /**
     * Returns a list of virtual machine we would like the new virtual machines to not be
     * co-located with (meaning not running on the same hardware resource).
     *
     * @return a list of virtual machines to no co-locate new ones with.
     */
    public List<VirtualMachine> separateFrom() {
        return Collections.unmodifiableList(noColocationList);
    }
}
